package meerkat.bulletinboard.sqlserver;

import meerkat.protobuf.BulletinBoardAPI.FilterType;

import java.util.LinkedList;
import java.util.List;

/**
 * Created by Arbel Deutsch Peled on 09-Dec-15.
 */

public class MySQLQueryProvider implements BulletinBoardSQLServer.SQLQueryProvider {

    String dbConnectionString;

    public MySQLQueryProvider(String dbAddress, String username, String password) {
        dbConnectionString = "jdbc:mysql:" + dbAddress + "?user=" + username + "&password=" + password;
    }

    @Override
    public String getSQLString(QueryType queryType) throws IllegalArgumentException{

        switch(queryType) {
            case ADD_SIGNATURE:
                return "INSERT IGNORE INTO SignatureTable (EntryNum, SignerId, Signature) VALUES (?,?,?)";
            case CONNECT_TAG:
                return "INSERT IGNORE INTO MsgTagTable (TagId, EntryNum)"
                    + " SELECT TagTable.TagId, ? AS EntryNum FROM TagTable WHERE Tag = ?";
            case FIND_MSG_ID:
                return "SELECT EntryNum From MsgTable WHERE MsgId = ?";
            case GET_MESSAGES:
                return "SELECT MsgTable.EntryNum, MsgTable.Msg FROM MsgTable";
            case GET_SIGNATURES:
                return "SELECT Signature FROM SignatureTable WHERE EntryNum = ?";
            case INSERT_MSG:
                return "INSERT INTO MsgTable (MsgId, Msg) VALUES(?,?)";
            case INSERT_NEW_TAG:
                return "INSERT IGNORE INTO TagTable(Tag) VALUES (?)";
           default:
               throw new IllegalArgumentException("Cannot serve a query of type " + queryType);
        }

    }

    @Override
    public String getCondition(FilterType filterType) throws IllegalArgumentException {

        switch(filterType) {
            case EXACT_ENTRY:
                return "MsgTable.EntryNum = ?";
            case MAX_ENTRY:
                return "MsgTable.EntryNum <= ?";
            case MAX_MESSAGES:
                return "LIMIT ?";
            case MSG_ID:
                return "MsgTable.MsgId = ?";
            case SIGNER_ID:
                return "EXISTS (SELECT 1 FROM SignatureTable"
                        + " WHERE SignatureTable.SignerId = ? AND SignatureTable.EntryNum = MsgTable.EntryNum)";
            case TAG:
                return "EXISTS (SELECT 1 FROM TagTable"
                        + " INNER JOIN MsgTagTable ON TagTable.TagId = MsgTagTable.TagId"
                        + " WHERE TagTable.Tag = ? AND MsgTagTable.EntryNum = MsgTable.EntryNum)";
            default:
                throw new IllegalArgumentException("Cannot serve a filter of type " + filterType);
        }

    }

    @Override
    public String getConnectionString() {
        return dbConnectionString;
    }


    @Override
    public List<String> getSchemaCreationCommands() {
        List<String> list = new LinkedList<String>();

        list.add("CREATE TABLE IF NOT EXISTS MsgTable (EntryNum INT NOT NULL AUTO_INCREMENT PRIMARY KEY, MsgId TINYBLOB, Msg BLOB, UNIQUE(MsgId(50)))");

        list.add("CREATE TABLE IF NOT EXISTS TagTable (TagId INT NOT NULL AUTO_INCREMENT PRIMARY KEY, Tag varchar(50), UNIQUE(Tag))");

        list.add("CREATE TABLE IF NOT EXISTS MsgTagTable (EntryNum INT, TagId INT,"
                + " CONSTRAINT FOREIGN KEY (EntryNum) REFERENCES MsgTable(EntryNum),"
                + " CONSTRAINT FOREIGN KEY (TagId) REFERENCES TagTable(TagId),"
                + " CONSTRAINT UNIQUE (EntryNum, TagID))");

        list.add("CREATE TABLE IF NOT EXISTS SignatureTable (EntryNum INT, SignerId TINYBLOB, Signature TINYBLOB, UNIQUE(Signature(150)),"
                + " INDEX(SignerId(50)), CONSTRAINT FOREIGN KEY (EntryNum) REFERENCES MsgTable(EntryNum))");

        return list;
    }

    @Override
    public List<String> getSchemaDeletionCommands() {
        List<String> list = new LinkedList<String>();

        list.add("DROP TABLE IF EXISTS MsgTagTable");
        list.add("DROP TABLE IF EXISTS SignatureTable");
        list.add("DROP TABLE IF EXISTS TagTable");
        list.add("DROP TABLE IF EXISTS MsgTable");

        return list;
    }
}
